# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime
import logging
import xml.etree.cElementTree as ET

from collections import OrderedDict
from driverng import MessageV2, utils
from driverng import Worklist

logger = logging.getLogger("driverng.drivers.mylaxml_v2")


class Driver(MessageV2):
    SPEC = {"code": "mylaxml_v2", "label": "Myla XML V2"}
    ENCODING = "utf8"

    OPTIONS = [
        MessageV2.Option("myla_v2_bmr", label="Champ bactérie multirésistante"),
        MessageV2.Option("myla_v2_default_cmi", label="CMI par défaut", default="-"),
        MessageV2.BooleanOption(
            "myla_v2_ignore_unk",
            label="Ignorer les résultats de type unk en hémoculture",
            default=False,
        ),
        MessageV2.BooleanOption(
            "myla_v2_hemoc_temps_positivite",
            label="Récupérer le délai uniquement pour les HEMOC +",
            default=False,
        ),
        MessageV2.Option(
            "myla_v2_hemoc_delai_incubation",
            label="Saisir le délai par défaut d'incubation pour l'hémoculture",
            default="",
        ),
        MessageV2.Option(
            "myla_v2_prefixe_germe",
            label="Préfixe des textes codés des germes (par défaut G)",
            default="G",
        ),
        MessageV2.Option(
            "myla_v2_prefixe_antibio",
            label="Préfixe des textes codés des antibiotiques (par défaut A)",
            default="A",
        ),
        MessageV2.BooleanOption(
            "myla_v2_atb_commentaire_automatique",
            label="Récupérer les commentaires automatisés (balise xml userAstCommentList)",
            default=False,
        ),
        MessageV2.BooleanOption(
            "myla_v2_germe_no_upper",
            label="Ne pas mettre le code du germe en majuscule",
            default=False,
        ),
        MessageV2.BooleanOption(
            "myla_v2_antibiotic_no_upper",
            label="Ne pas mettre les codes des antibiotiques en majuscule",
            default=False,
        ),
        MessageV2.BooleanOption(
            "myla_v2_ignore_suppressed_drug",
            label="Ignorer les antibiotiques supprimés (balise suppressedDrug)",
            default=False,
        ),
        MessageV2.BooleanOption(
            "myla_v2_send_service_xml_location",
            label="Envoyer le code du service dans la balise xml location",
            default=False,
        ),
        MessageV2.Option(
            "myla_v2_prefixe_atb_adagio",
            label="Préfixe pour les antibiotiques réalisés par l'ADAGIO (via MYLA)",
            default="",
        ),
        MessageV2.BooleanOption(
            "myla_v2_diameter",
            label="Intégrer les diamètres dans les ATB",
            default=False,
        ),
        MessageV2.BooleanOption(
            "myla_v2_commentaire_user",
            label="Récupérer les commentaires utilisateur (balise xml userComment/comment/text)",
            default=False,
        ),
        MessageV2.BooleanOption(
            "myla_v2_instrument_name",
            label="Récupérer instrumentName au lieu de serialNumber dans le champ analyseur",
            default=False,
        ),
        MessageV2.BooleanOption(
            "myla_v2_send_atb_to_myla",
            label="Envoyer les antibiogrammes vers le Myla",
            default=False,
        ),
    ]

    def __init__(self, **kwargs):
        self.myla_v2_bmr = kwargs.pop("myla_v2_bmr", "")
        self.myla_v2_default_cmi = kwargs.pop("myla_v2_default_cmi", "-")
        self.myla_v2_ignore_unk = kwargs.pop("myla_v2_ignore_unk", False)
        self.myla_v2_hemoc_temps_positivite = kwargs.pop(
            "myla_v2_hemoc_temps_positivite", False
        )
        self.myla_v2_hemoc_delai_incubation = kwargs.pop(
            "myla_v2_hemoc_delai_incubation", ""
        )
        self.myla_v2_prefixe_germe = kwargs.pop("myla_v2_prefixe_germe", "G")
        self.myla_v2_prefixe_antibio = kwargs.pop("myla_v2_prefixe_antibio", "A")
        self.myla_v2_atb_commentaire_automatique = kwargs.pop(
            "myla_v2_atb_commentaire_automatique", False
        )
        self.myla_v2_germe_no_upper = kwargs.pop("myla_v2_germe_no_upper", False)
        self.myla_v2_antibiotic_no_upper = kwargs.pop(
            "myla_v2_antibiotic_no_upper", False
        )
        self.myla_v2_ignore_suppressed_drug = kwargs.pop(
            "myla_v2_ignore_suppressed_drug", False
        )
        self.myla_v2_send_service_xml_location = kwargs.pop(
            "myla_v2_send_service_xml_location", False
        )
        self.myla_v2_prefixe_atb_adagio = kwargs.pop("myla_v2_prefixe_atb_adagio", "")
        self.myla_v2_diameter = kwargs.pop("myla_v2_diameter", False)
        self.myla_v2_commentaire_user = kwargs.pop("myla_v2_commentaire_user", False)
        self.myla_v2_instrument_name = kwargs.pop("myla_v2_instrument_name", False)
        self.myla_v2_send_atb_to_myla = kwargs.pop("myla_v2_send_atb_to_myla", False)
        return super(Driver, self).__init__(**kwargs)

    def parse(self, data):
        return ET.fromstring(data.encode("utf8"))

    def to_string(self, message):
        return utils.prettify_xml(ET.tostring(message, "utf-8"))

    """
    Construction du bloc :
    <testOrder>
        <specimen>
        ...
        </specimen>
    """

    @staticmethod
    def get_test_order(request, sample):
        test_order = ET.SubElement(request, "testOrder")
        specimen = ET.SubElement(test_order, "specimen")
        ET.SubElement(specimen, "specimenIdentifier").text = sample.sample_id

        for order in sample.orders:
            ET.SubElement(specimen, "specimenType").text = order.code
            ET.SubElement(specimen, "specimenTypeText").text = order.code
            ET.SubElement(specimen, "specimenNature").text = order.code
            ET.SubElement(specimen, "specimenNatureText").text = order.code
            if sample.collection_date:
                ET.SubElement(specimen, "collectedDateTime").text = utils.format_date(
                    sample.collection_date, fmt="%Y%m%d%H%M%S"
                )
        return test_order

    def encode_worklist(self, worklist, context):
        root = ET.Element("lisMessage")
        header = ET.SubElement(root, "header")
        ET.SubElement(header, "version").text = "1394-97"
        ET.SubElement(header, "dateTime").text = utils.format_date(
            context["now"], fmt="%Y%m%d%H%M%S"
        )

        empty = True
        for sample, _ in worklist.get_worklist(
            unique_together=["action"],
            action={"del": worklist.IGNORE, worklist.ELSE: None},
        ):
            request = ET.SubElement(root, "request")
            patient = ET.SubElement(request, "patientInformation")

            # Identité du patient
            ET.SubElement(patient, "patientIdentifier").text = sample.patient_id
            if sample.alternate_patient_id:
                ET.SubElement(
                    patient, "patientVisitNumber"
                ).text = sample.alternate_patient_id
            ET.SubElement(patient, "firstName").text = sample.first_name
            ET.SubElement(patient, "lastName").text = sample.last_name
            if sample.birthdate:
                ET.SubElement(patient, "birthdate").text = utils.format_date(
                    sample.birthdate, fmt="%Y%m%d"
                )
            ET.SubElement(patient, "sex").text = sample.sex

            # Médecin
            physician = [p for p in sample.physicians if p.type.lower() == "prescriber"]
            if physician:
                ET.SubElement(patient, "physicianIdentifier").text = physician[0].code
                ET.SubElement(patient, "physicianName").text = physician[0].last_name

            # Service et admission
            if sample.admission_date:
                ET.SubElement(patient, "admissionDate").text = utils.format_date(
                    sample.admission_date, fmt="%Y%m%d"
                )
            if sample.room and not self.myla_v2_send_service_xml_location:
                ET.SubElement(patient, "location").text = sample.room
                ET.SubElement(patient, "locationText").text = sample.room
            elif sample.location and self.myla_v2_send_service_xml_location:
                ET.SubElement(patient, "location").text = sample.location
                ET.SubElement(patient, "locationText").text = sample.location
            if sample.hospital_service:
                ET.SubElement(patient, "hospitalService").text = sample.hospital_service
                ET.SubElement(
                    patient, "hospitalServiceText"
                ).text = sample.hospital_service
            elif sample.location:
                ET.SubElement(patient, "hospitalService").text = sample.location
                if sample.location_name:
                    ET.SubElement(
                        patient, "hospitalServiceText"
                    ).text = sample.location_name
                else:
                    ET.SubElement(patient, "hospitalServiceText").text = sample.location
            if sample.lab:
                ET.SubElement(patient, "hospitalInstitution").text = sample.lab
                ET.SubElement(patient, "hospitalInstitutionText").text = sample.lab

            # Mise en place du traitement des résultats selon leur nature
            identification_germ_to_send = False
            for result in sample.results:
                if result.resu_json and "germes" in result.resu_json:
                    identification_germ_to_send = True

            # Si il y a des résultats d'identification de germe et/ou d'antibiogramme à envoyer. Il faut envoyer un spécimen et un résultat par testOrder
            if identification_germ_to_send or self.myla_v2_send_atb_to_myla:

                for result in sample.results:
                    if isinstance(result.value, list):
                        # Envoi de résultats contenant un antibiogramme
                        if self.myla_v2_send_atb_to_myla and result.value[0].get(
                            "antibiogram"
                        ):

                            test_order = self.get_test_order(request, sample)

                            rank_germ = result.sub_code
                            id_germ = result.value[0].get("value")
                            ET.SubElement(test_order, "isolateNumber").text = str(
                                rank_germ if rank_germ is None else ""
                            )
                            test = ET.SubElement(test_order, "test")
                            universal_identifier = ET.SubElement(
                                test, "universalIdentifier"
                            )

                            # Identification de germe
                            if id_germ:
                                ET.SubElement(
                                    universal_identifier, "testGroupCode"
                                ).text = "ID"
                                resu = ET.SubElement(test, "result")
                                value = ET.SubElement(resu, "value")
                                identification = ET.SubElement(value, "identification")
                                significant_taxon = ET.SubElement(
                                    identification, "significantTaxon"
                                )
                                germ_prefix = "{}_".format(self.myla_v2_prefixe_germe)
                                ET.SubElement(significant_taxon, "code").text = (
                                    id_germ[2:]
                                    if id_germ
                                    and germ_prefix
                                    and id_germ.startswith(germ_prefix)
                                    else id_germ
                                )
                                ET.SubElement(
                                    significant_taxon, "name"
                                ).text = result.value_label

                            # Antibiogramme
                            ast = ET.SubElement(value, "ast")
                            for atb in result.value[0].get("antibiogram"):
                                antibiotic = ET.SubElement(ast, "antibiotic")
                                code_atb = atb.get("antibiotic")
                                ET.SubElement(antibiotic, "code").text = (
                                    code_atb[2:]
                                    if code_atb.startswith("A_")
                                    else code_atb
                                )
                                antibiotic_name = ET.SubElement(antibiotic, "name")
                                antibiotic_texte_code = atb.get("antibiotic_texte_code")
                                if antibiotic_texte_code:
                                    antibiotic_name.text = antibiotic_texte_code[1]
                                highmic = ET.SubElement(antibiotic, "highMic")
                                if atb.get("mic"):
                                    cmi = atb.get("mic")
                                    if (cmi.startswith("<=") and cmi[:2] == "<=") or (
                                        cmi.startswith(">=") and cmi[:2] == ">="
                                    ):
                                        ET.SubElement(highmic, "micSign").text = cmi[:2]
                                        cmi = cmi[2:]
                                    elif (cmi.startswith(">") and cmi[:2] != ">=") or (
                                        cmi.startswith("<") and cmi[:2] != "<="
                                    ):
                                        ET.SubElement(highmic, "micSign").text = cmi[:1]
                                        cmi = cmi[1:]
                                        ET.SubElement(highmic, "micValue").text = cmi
                                ET.SubElement(antibiotic, "category").text = atb.get(
                                    "sensitivity", ""
                                )
                                ET.SubElement(
                                    antibiotic, "nonExpertCategory"
                                ).text = atb.get("sensitivity", "")
                            empty = False

                    # Pas d'antibiogramme, envoi seulement de l'identification
                    elif (
                        identification_germ_to_send
                        and result.resu_json
                        and "germes" in result.resu_json
                    ):
                        for resu_json_germe in result.resu_json.germes:
                            if not isinstance(resu_json_germe, dict):
                                continue

                            test_order = self.get_test_order(request, sample)

                            try:
                                rank_germ = resu_json_germe.get("rang")
                            except KeyError:
                                rank_germ = ""
                            try:
                                key_germe = [
                                    cle
                                    for cle in resu_json_germe.keys()
                                    if cle.lower() == "id"
                                ]
                                id_germ = resu_json_germe.get(
                                    key_germe[0] if key_germe else None
                                )
                                if isinstance(id_germ, list):
                                    id_germ = ""
                            except KeyError:
                                id_germ = ""

                            # Rang du germe identifié
                            if id_germ and rank_germ:
                                ET.SubElement(test_order, "isolateNumber").text = str(
                                    rank_germ
                                )
                                rank_germ = ""
                            test = ET.SubElement(test_order, "test")
                            universal_identifier = ET.SubElement(
                                test, "universalIdentifier"
                            )

                            # Identification du germe
                            if id_germ:
                                ET.SubElement(
                                    universal_identifier, "testGroupCode"
                                ).text = "ID"
                                resu = ET.SubElement(test, "result")
                                value = ET.SubElement(resu, "value")
                                identification = ET.SubElement(value, "identification")
                                significant_taxon = ET.SubElement(
                                    identification, "significantTaxon"
                                )
                                germ_prefix = "{}_".format(self.myla_v2_prefixe_germe)
                                ET.SubElement(significant_taxon, "code").text = (
                                    id_germ[2:]
                                    if id_germ
                                    and germ_prefix
                                    and id_germ.startswith(germ_prefix)
                                    else id_germ
                                )
                                id_germ = ""
                            empty = False

            # Pas de résultats d'identification de germe ni d'antibiogramme - Hémoculture
            else:
                for order in sample.orders:
                    test_order = ET.SubElement(request, "testOrder")
                    specimen = ET.SubElement(test_order, "specimen")
                    specimen_nature = ""
                    specimen_source = ""
                    for result in sample.results:
                        if result.code.upper().startswith("SPECIMENNATURE_"):
                            specimen_nature = result.value
                        elif result.code.upper().startswith("SPECIMENSOURCE_"):
                            specimen_source = result.value
                    ET.SubElement(
                        specimen, "specimenIdentifier"
                    ).text = sample.sample_id

                    # Code ou Type de prélèvement
                    prelevement_type = order.code
                    delai_incubation_hemoculture = False
                    for result in sample.results:
                        if sample.specimen == "BC" and result.code.startswith(
                            "EXPECTED_DURATION"
                        ):
                            delai_incubation_hemoculture = True
                            prelevement_type = sample.specimen
                            continue
                    if not delai_incubation_hemoculture:
                        tcode = order.code.upper().split("_")
                        if len(tcode) > 1:
                            prelevement_type = tcode[1]
                    if prelevement_type:
                        ET.SubElement(specimen, "specimenType").text = prelevement_type
                        ET.SubElement(
                            specimen, "specimenTypeText"
                        ).text = prelevement_type

                    # Nature du prélèvement
                    if specimen_nature:
                        ET.SubElement(specimen, "specimenNature").text = specimen_nature
                        ET.SubElement(
                            specimen, "specimenNatureText"
                        ).text = specimen_nature
                    else:
                        ET.SubElement(specimen, "specimenNature").text = order.code
                        ET.SubElement(specimen, "specimenNatureText").text = order.code

                    # Localisation ou source de prélèvement
                    if specimen_source:
                        ET.SubElement(specimen, "specimenSource").text = specimen_source
                        ET.SubElement(
                            specimen, "specimenSourceText"
                        ).text = specimen_source
                    if sample.collection_date:
                        ET.SubElement(
                            specimen, "collectedDateTime"
                        ).text = utils.format_date(
                            sample.collection_date, fmt="%Y%m%d%H%M%S"
                        )

                    test = ET.SubElement(test_order, "test")
                    universal_identifier = ET.SubElement(test, "universalIdentifier")

                    # Type de prélèvement d'hémoculture
                    if sample.specimen == "SU":
                        ET.SubElement(
                            universal_identifier, "testGroupCode"
                        ).text = sample.specimen
                    elif sample.specimen == "ID":
                        ET.SubElement(
                            universal_identifier, "testGroupCode"
                        ).text = sample.specimen
                    elif sample.specimen == "BC":
                        ET.SubElement(
                            universal_identifier, "testGroupCode"
                        ).text = sample.specimen
                        delai_duration = False
                        for result in sample.results:
                            # Transmettre uniquement le premier test de délai d'incubation
                            if result.code.startswith("EXPECTED_DURATION"):
                                ET.SubElement(
                                    test, "expectedDuration"
                                ).text = result.value
                                delai_duration = True
                                continue
                        if self.myla_v2_hemoc_delai_incubation and not delai_duration:
                            ET.SubElement(
                                test, "expectedDuration"
                            ).text = self.myla_v2_hemoc_delai_incubation
                    empty = False
        if not empty:
            return root

    def decode_message(self, message):
        samples_results = OrderedDict()
        worklist = Worklist(context="result")

        # Instrument
        if message.find("header"):
            if (
                message.find("header").find("serialNumber") is None
                and not self.myla_v2_instrument_name
            ):
                analyseur = ""
            elif not self.myla_v2_instrument_name:
                analyseur = message.find("header").find("serialNumber").text
            else:
                analyseur = message.find("header").find("instrumentName").text
        else:
            analyseur = ""

        for request in message.findall("requestResult"):
            for test_order in request.findall("testOrder"):
                sample_id = test_order.findtext("./specimen/specimenIdentifier")
                sample_results = samples_results.setdefault(sample_id, OrderedDict())
                specimen_type = test_order.findtext("./specimen/specimenType")

                # Flacon d'hémoculture
                if (
                    test_order.findtext(
                        "./test/result/bmxResultData/disposable/disposableType"
                    )
                    == "BOTTLE"
                ):
                    reference = test_order.findtext(
                        "./test/result/bmxResultData/disposable/reference"
                    )
                    instrument_name = "{:.20s}".format(
                        test_order.findtext("./test/instrumentName")
                    )
                    date = datetime.datetime.strptime(
                        test_order.findtext(
                            "./test/result/value/bloodCulture/dateResultDetermined"
                        ),
                        "%Y%m%d%H%M%S",
                    )

                    # Section ID
                    code = "{}_SECTION-ID".format(reference)
                    sample_results[code] = {
                        "code": code,
                        "value": test_order.findtext("./test/instrumentSectionId"),
                        "date": date,
                        "instrument": instrument_name,
                    }
                    # Identification
                    code = "{}_IDENT".format(reference)
                    sample_results[code] = {
                        "code": code,
                        "value": test_order.findtext(
                            "./test/result/bmxResultData/disposable/disposableIdentifier"
                        ),
                        "date": date,
                        "instrument": instrument_name,
                    }
                    # Volume
                    volume = test_order.findtext("./test/result/bmxResultData/volume")
                    if volume:
                        code = "{}_VOLUME".format(reference)
                        sample_results[code] = {
                            "code": code,
                            "value": volume,
                            "date": date,
                            "instrument": instrument_name,
                        }
                    # Dates
                    for suffix, value in (
                        (
                            "_START_DATETIME",
                            test_order.findtext("./test/result/startDateTime"),
                        ),
                        (
                            "_DISPOSABLE_START_DATETIME",
                            test_order.findtext(
                                "./test/result/bmxResultData/disposable/disposableStartDateTime"
                            ),
                        ),
                        (
                            "_DISPOSABLE_FINAL_DATETIME",
                            test_order.findtext(
                                "./test/result/bmxResultData/disposable/disposableFinalDateTime"
                            ),
                        ),
                    ):
                        if value is not None:
                            code = "{}{}".format(reference, suffix)
                            sample_results[code] = {
                                "code": code,
                                "value": value,
                                "instrument": instrument_name,
                            }
                    # Flacon
                    sample_results["BOTTLE_TYPE"] = {
                        "code": "BOTTLE_TYPE",
                        "value": reference,
                        "instrument": instrument_name,
                    }

                    # Ignoré dans les messages d'acquittement du tube
                    if test_order.findtext("./test/result/resultStatus") != "I":
                        # Résultats d'hémoculture
                        value = test_order.findtext(
                            "./test/result/value/bloodCulture/result"
                        )
                        # Si l'option est activée, on ignore le résultat d'hémoculture de type unk
                        # Le Myla envoie ce type de résultat lorsqu'il ne sait pas donner un résultat. Impossible sur le MYLA de ne pas envoyer ce résultat
                        if self.myla_v2_ignore_unk and value == "Unk":
                            continue
                        operator = test_order.findtext(
                            "./test/result/bmxResultData/disposable/disposableSetupTechnologist"
                        )
                        code = "{}_RESULT".format(reference)
                        sample_results[code] = {
                            "code": code,
                            "value": value,
                            "instrument": instrument_name,
                            "operator": operator if operator else "",
                            "date": date,
                        }
                        # Flags
                        if test_order.findtext("./test/result/abnormalFlags"):
                            code = "{}_ABNORMAL_FLAG".format(reference)
                            sample_results[code] = {
                                "code": code,
                                "value": test_order.findtext(
                                    "./test/result/abnormalFlags"
                                ),
                                "date": date,
                                "instrument": instrument_name,
                            }
                        # Dates
                        for suffix, value in (
                            (
                                "_TIME_TO_DETECTION",
                                test_order.findtext(
                                    "./test/result/value/bloodCulture/timeToDetectionInHours"
                                ),
                            ),
                            (
                                "_DETERMINED_DATETIME",
                                test_order.findtext(
                                    "./test/result/value/bloodCulture/dateResultDetermined"
                                ),
                            ),
                        ):
                            if (
                                self.myla_v2_hemoc_temps_positivite
                                and suffix == "_TIME_TO_DETECTION"
                                and test_order.findtext(
                                    "./test/result/value/bloodCulture/result"
                                )
                                != "Pos"
                            ):
                                continue
                            if suffix == "_TIME_TO_DETECTION":
                                try:
                                    value_in_minutes = float(value) * 60
                                except ValueError:
                                    logger.warning(
                                        'Le délai dans la balise "timeToDetectionInHours" n\'est pas une valeur numérique'
                                    )
                                    continue
                                code = "{}_TIME_TO_DETECTION_POSITIVITY".format(
                                    reference
                                )
                                sample_results[code] = {
                                    "code": code,
                                    "value": "{:02d}:{:02d}".format(
                                        int(value_in_minutes // 60),
                                        int(value_in_minutes % 60),
                                    ),
                                    "instrument": instrument_name,
                                }
                            if value is not None:
                                code = "{}{}".format(reference, suffix)
                                sample_results[code] = {
                                    "code": code,
                                    "value": value,
                                    "instrument": instrument_name,
                                }

                elif specimen_type:
                    if self.myla_v2_germe_no_upper:
                        # Ne pas mettre le code du germe en majuscule
                        germe = "{}_{}".format(
                            self.myla_v2_prefixe_germe,
                            test_order.findtext(
                                "./test/result/value/identification/significantTaxon/code"
                            ),
                        )
                    else:
                        # Mise en majuscule du code du germe. Comportement par
                        # défaut afin de conserver la compatibilité avec les
                        # clients ayant les textes codés des germes paramétrés
                        # en majuscule
                        germe = "{}_{}".format(
                            self.myla_v2_prefixe_germe,
                            test_order.findtext(
                                "./test/result/value/identification/significantTaxon/code"
                            ).upper(),
                        )
                    # Traitement des commentaires automatisés des antibiogrammes
                    ast_comments = []
                    user_comments = []
                    phenotypes = []
                    for resistance_mechanism in test_order.findall(
                        "./test/result/value/expertFinding/resistanceMechanism"
                    ):
                        drug_family = resistance_mechanism.findtext("drugFamily")
                        for phenotype in resistance_mechanism.findall("phenotype"):
                            drug_family_phenotype = "{}-{}".format(
                                drug_family, phenotype.text
                            )
                            phenotypes.append(drug_family_phenotype)
                    for ast_comment in test_order.findall(
                        "./test/result/value/ast/userAstCommentList/comment"
                    ):
                        if self.myla_v2_atb_commentaire_automatique:
                            ast_comments.append(ast_comment.find(".").text)
                    if self.myla_v2_commentaire_user:
                        for user_comment in test_order.findall(
                            "./test/result/userComment/comment"
                        ):
                            com_texte = user_comment.findtext("text")
                            if com_texte:
                                user_comments.append(com_texte)
                    seq = test_order.findtext("isolateNumber")
                    result = {
                        "code": specimen_type,
                        "value": [{"antibiogram": [], "sub_code": seq, "value": germe}],
                        "date": datetime.datetime.strptime(
                            test_order.findtext("./test/result/resultDateTime"),
                            "%Y%m%d%H%M%S",
                        ),
                        "phenotypes": phenotypes,
                        "comments": user_comments or ast_comments,
                        "instrument": analyseur,
                        "nom_antibiogramme": test_order.findtext(
                            "./test/universalIdentifier/testIdentifier"
                        ),
                    }
                    # Gestion des bactéries multi résistantes
                    # On le fait passer en antibiotique dans le même test même si ça n'a rien à voir
                    for resistance in test_order.findall(
                        "./test/result/value/expertFinding/resistanceMechanism"
                    ):
                        if resistance.findtext("drugFamily") == self.myla_v2_bmr:
                            bmrs = [e.text for e in resistance.findall("phenotype")]
                            # Spécifique Labomaine
                            if (
                                germe != "G_STAAUR"
                                and "MODIFICATION DES PLP (mecA)" in bmrs
                            ):
                                bmrs.remove("MODIFICATION DES PLP (mecA)")
                            if bmrs:
                                result.get("value")[0].get("antibiogram").append(
                                    {
                                        "antibiotic": ";".join(
                                            ["BMR_%s" % (e,) for e in bmrs]
                                        ),
                                        "mic": "#",
                                        "sensitivity": "BMR",
                                    }
                                )
                    # Réponse automate pour les ast_comments
                    if ast_comments:
                        sample_results["ATB_BIOART"] = {
                            "code": "ATB_BIOART",
                            "value": ast_comments,
                        }

                    # Traitement des antibiotiques
                    for antibiotic in test_order.findall(
                        "./test/result/value/ast/antibiotic"
                    ):
                        if (
                            self.myla_v2_ignore_suppressed_drug
                            and antibiotic.findtext("suppressedDrug")
                            and antibiotic.find("suppressedDrug").text == "true"
                        ):
                            continue
                        if self.myla_v2_antibiotic_no_upper:
                            antibiotic_code = antibiotic.findtext("code")
                        else:
                            antibiotic_code = antibiotic.findtext("code").upper()
                        if analyseur == "ADAGIO" and self.myla_v2_prefixe_atb_adagio:
                            code = "{}_{}".format(
                                self.myla_v2_prefixe_atb_adagio, antibiotic_code
                            )
                        elif self.myla_v2_prefixe_antibio:
                            code = "{}_{}".format(
                                self.myla_v2_prefixe_antibio, antibiotic_code
                            )
                        else:
                            code = antibiotic_code
                        cmi = "{}{}".format(
                            antibiotic.findtext("./highMic/micSign") or "",
                            antibiotic.findtext("./highMic/micValue") or "",
                        )
                        sensi = antibiotic.findtext("category")
                        # Ignorer les résultats de cmi et de sensibilité non testés
                        if cmi == "(-)" or sensi == "(-)":
                            continue
                        if self.myla_v2_diameter and antibiotic.findtext("diameter"):
                            result.get("value")[0].get("antibiogram").append(
                                {
                                    "antibiotic": code,
                                    "mic": cmi or self.myla_v2_default_cmi,
                                    "sensitivity": sensi,
                                    "diameter": antibiotic.findtext("diameter"),
                                }
                            )
                        else:
                            result.get("value")[0].get("antibiogram").append(
                                {
                                    "antibiotic": code,
                                    "mic": cmi or self.myla_v2_default_cmi,
                                    "sensitivity": sensi,
                                }
                            )

                    if not result.get("value")[0].get("antibiogram"):
                        # Identification des germes
                        code = "GERME_{}_{}".format(seq, specimen_type)
                        sample_results[code] = {
                            "code": code,
                            "value": germe,
                            "instrument": analyseur,
                        }

                        result["code"] = "IDENT_{}".format(result["code"])
                        result["value"] = {
                            "type_resultat": "identification_germes",
                            "germes": [],
                            "seq": "0",
                        }
                        if result["code"] in sample_results:
                            result["value"]["germes"] = sample_results[result["code"]][
                                "value"
                            ].get("germes", [])
                        result["value"]["germes"].append(
                            {
                                "id": germe,
                                "rang": int(test_order.findtext("isolateNumber")),
                            }
                        )
                    elif result["code"] in sample_results:
                        raise NotImplementedError(
                            "Plusieurs résultats pour un même test avec ATB."
                        )
                    sample_results[result["code"]] = result

        for sample_id, data in samples_results.items():
            sample = worklist.get_or_create(sample_id=sample_id, type="patient")
            for key in data:
                sample.results.add(data.get(key))

        return worklist
